/*
 * The Program materials contained in this file are IBM copyright materials.
 * COPYRIGHT International Business Machines Corp. 2015
 * All Rights Reserved * Licensed Materials - Property of IBM
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

var globalization = (function() {
	
	var bidiEnabled = false;
	var bidiTextDirection = "ltr";
	
	var supportedLanguages = ["cs",
							  "de",
							  "es", 
							  "fr",
							  "hu",
							  "it",
							  "ja",
							  "ko",
							  "pl",
							  "pt-br",
							  "ro",
							  "ru",
							  "zh",
							  "zh-tw"
							 ];


	var retrieveExternalizedStrings = function() {
		
		// Obtain browser language
		var browserLanguage = navigator.languages? navigator.languages[0] : (navigator.language || navigator.userLanguage);
		
		// Ensure lower case is used
		browserLanguage = browserLanguage.toLowerCase();
		
		// language code for translations retrieval
		var languageCode = "";
		if(supportedLanguages.indexOf(browserLanguage) != -1) {
			languageCode = browserLanguage;
		} else if(browserLanguage.length == 5) {
			var code = browserLanguage.substring(0, 2);
			if(supportedLanguages.indexOf(code) != -1) {
				languageCode = code;
			}
		}
		var url = languageCode ? "nls/" + languageCode + "/messages.js" : "nls/messages.js";
			
		// Retrieve translations
		return $.ajax({
			url: url,
			dataType: "script",
			success: function() {
				replaceExternalizedStrings();
			}
		});
	};
	
	
	var retrieveBidiPreference = function() {
		return $.ajax({
			url: "/ibm/api/adminCenter/v1/toolbox/preferences",
			success: function(data) {
				if(data) {
					bidiEnabled = data.bidiEnabled;
					bidiTextDirection = data.bidiTextDirection;
				}
			}
		});
	};

	
	var dataTypeRequiresSpecialHandling = function(dataType) {
		return dataType == "location" || dataType == "dirLocation" || dataType == "fileLocation";
	};
	
	
	var replaceExternalizedStrings = function() {
		
		$("[data-externalizedString]").each(function() {
			var element = $(this);
			element.append(editorMessages[element.attr("data-externalizedString")]);
		});
		
		$("[data-externalizedPlaceholder]").each(function() {
			var element = $(this);
			element.attr("placeholder", editorMessages[element.attr("data-externalizedPlaceholder")]);
		});
		
		$("[data-externalizedStringTitle]").each(function() {
			var element = $(this);
			element.attr("title", editorMessages[element.attr("data-externalizedStringTitle")]);
		});
		
		$("[data-externalizedStringAlt]").each(function() {
			var element = $(this);
			element.attr("alt", editorMessages[element.attr("data-externalizedStringAlt")]);
		});

	};
	
	var obtainContextualDir = function(text) {
		// look for strong (directional) characters
		var fdc = /[A-Za-z\u05d0-\u065f\u066a-\u06ef\u06fa-\u07ff\ufb1d-\ufdff\ufe70-\ufefc]/.exec(text);
		// if found return the direction that defined by the character, else return "ltr"
		return fdc ? ( fdc[0] <= 'z' ? "ltr" : "rtl" ) : "ltr";
	};

	
	return {
		retrieveExternalizedStrings: retrieveExternalizedStrings,
		retrieveBidiPreference: retrieveBidiPreference,
		obtainContextualDir: obtainContextualDir,
		dataTypeRequiresSpecialHandling: dataTypeRequiresSpecialHandling,
		isBidiEnabled: function() {
			return bidiEnabled;
		},
		getBidiTextDirection: function() {
			return bidiTextDirection;
		}
	};
	
})();
