/*
 * The Program materials contained in this file are IBM copyright materials.
 * COPYRIGHT International Business Machines Corp. 2015
 * All Rights Reserved * Licensed Materials - Property of IBM
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

var saveBeforeClosingDialog = (function() {
	
	var callbackFunction = null;
	
	var invokeCallbackFunction = function() {
		if(callbackFunction != null) {
			callbackFunction();
		}
	};
	
	return {

		setCallbackFunction: function(newFunction) {
			callbackFunction = newFunction;
		},
		
		getCallbackFunction: function() {
			return callbackFunction;
		},
		
		invokeCallbackFunction: invokeCallbackFunction
	
	};

})();


$(document).ready(function() {
	
	$("#dialogSaveBeforeClosing").on("show.bs.modal", function(event) {
		
		// Update file name
		var fileName = $("#navigationBarTitle").text();
		$("#dialogSaveBeforeClosingText").text(stringUtils.formatString(editorMessages.SAVE_BEFORE_CLOSING_DIALOG_MESSAGE, [fileName]));
		
		// Update button states
		var otherButtons = $("#dialogSaveBeforeClosing .dialog-close-link, #dialogSaveBeforeClosingDontSaveButton");
		otherButtons.removeAttr("disabled");
		
		// Update button label
		$("#dialogSaveBeforeClosingSaveButton").text(editorMessages.SAVE).removeClass("loading");
		
		// Center dialog
		core.centerDialog(this);
	});


	$("#dialogSaveBeforeClosingSaveButton").on("vclick", function(event) {
		event.preventDefault();
		
		var button = $(this);
		
		// Update button label
		button.html("<img src=\"img/config-tool-home-progress-bgblue-D.gif\">" + editorMessages.SAVING).addClass("loading");
		
		// Disable close without saving and cancel buttons
		var otherButtons = $("#dialogSaveBeforeClosing .dialog-close-link, #dialogSaveBeforeClosingDontSaveButton");
		otherButtons.attr("disabled", "disabled");
		
		// Block UI
		core.setUIBlocked(true);
		
		editor.fileChangedDuringEditing().done(function() {
			
			// Pass callback function
			var callbackFunction = saveBeforeClosingDialog.getCallbackFunction();
			fileChangedDuringEditingDialog.setCallbackFunction(callbackFunction);
			
			$("#dialogSaveBeforeClosing").modal("hide");
			
			// Show overwrite dialog
			$("#dialogFileChangedDuringEditing").modal("show");
			
		}).fail(function() {
			
			editor.save().done(function() {
				saveBeforeClosingDialog.invokeCallbackFunction();
			}).fail(function() {
				
				// Show error message
				$("#dialogErrorSavingFile").modal("show");
				
			}).always(function() {
				$("#dialogSaveBeforeClosing").modal("hide");
				
			});
			
		}).always(function() {
			
			// Unblock UI
			core.setUIBlocked(false);
			
		});
		
		
	});
	
	
	$("#dialogSaveBeforeClosingDontSaveButton").on("vclick", function(event) {
		event.preventDefault();
		$("#dialogSaveBeforeClosing").modal("hide");
		saveBeforeClosingDialog.invokeCallbackFunction();
	});

	
});
