/*
 * The Program materials contained in this file are IBM copyright materials.
 * COPYRIGHT International Business Machines Corp. 2015
 * All Rights Reserved * Licensed Materials - Property of IBM
 * US Government Users Restricted Rights - Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

var fileUtils = (function() {

	var serverVariables = [
	{	name: "wlp.install.dir",
		resolvedPath: null
	},
	{	name: "wlp.user.dir",
		resolvedPath: null
	},
	{	name: "server.config.dir",
		resolvedPath: null
	},
	{	name: "server.output.dir",
		resolvedPath: null
	},
	{	name: "shared.app.dir",
		resolvedPath: null
	},
	{	name: "shared.config.dir",
		resolvedPath: null
	},
	{	name: "shared.resource.dir",
		resolvedPath: null
	} ];
	
	
	var serverWritePaths = null;
	

	var resolveIncludeLocations = function(contextLocation, includeLocations) {
		var deferred = new $.Deferred();
			var resolvedConfigurationFiles = [];
			var deferredCounter = 0;
			for(var i = 0; i < includeLocations.length; i ++) {
				deferredCounter++;
				var fileLocation = includeLocations[i];
				var resolveFunction = function(index, fileLocation) {
					resolvedConfigurationFiles[index] = {
							location: fileLocation
					};
					resolveIncludeLocation(contextLocation, fileLocation).done(function(metadata) {
						resolvedConfigurationFiles[index].resolvedLocation = metadata.fileName;
						resolvedConfigurationFiles[index].editable = metadata.readOnly == "false" && isPathWritable(metadata.fileName);
					}).fail(function() {
						resolvedConfigurationFiles[index].resolvedLocation = null;
						resolvedConfigurationFiles[index].editable = false;
					}).always(function() {
						deferredCounter--;
						if(deferredCounter == 0) {
							deferred.resolve(resolvedConfigurationFiles);
						}
					});
				};
				resolveFunction(i, fileLocation);
			}
		return deferred;
	};


	var resolveIncludeLocation = function(contextLocation, fileLocation) {
		var deferred = new $.Deferred();
		var retrieveFileMetadataPromise = retrieveFileMetadata(contextLocation + "/" + fileLocation).done(function(metadata) {
			deferred.resolve(metadata);
		}).fail(function(textStatus) {
			if(textStatus == "abort") return;
			retrieveFileMetadataPromise = retrieveFileMetadata("${server.config.dir}" + fileLocation).done(function(metadata) {
				deferred.resolve(metadata);
			}).fail(function(textStatus) {
				if(textStatus == "abort") return;
				retrieveFileMetadataPromise = retrieveFileMetadata("${shared.config.dir}" + fileLocation).done(function(metadata) {
					deferred.resolve(metadata);
				}).fail(function(textStatus) {
					if(textStatus == "abort") return;
					retrieveFileMetadataPromise = retrieveFileMetadata(fileLocation).done(function(metadata) {
						deferred.resolve(metadata);
					}).fail(function(textStatus) {
						if(textStatus == "abort") return;
						deferred.reject();
					});
				});
			});
		});
		deferred.abort = function() {
			retrieveFileMetadataPromise.abort();
		};
		return deferred;
	};

	
	var retrieveFile = function(filePath) {
		var deferred = new $.Deferred();
		retrieveFileMetadataPromise = retrieveFileMetadata(filePath);
		retrieveFileContentPromise = retrieveFileContent(filePath);
		$.when(retrieveFileMetadataPromise, retrieveFileContentPromise).done(function(fileMetadata, fileContent) {
			var file = {
				path: applyVariablesToFilePath(filePath),
				resolvedPath: fileMetadata.fileName,
				content: fileContent,
				isReadOnly: fileMetadata.readOnly == "true"? true : !isPathWritable(fileMetadata.fileName)
			};
			deferred.resolve(file);
		}).fail(function(error) {
			deferred.reject();
		});
		return deferred;
	};
	

	var retrieveFileContent = function(filePath) {
		var deferred = new $.Deferred();

		$.ajax({
			url: "/IBMJMXConnectorREST/file/" + encodeURIComponent(filePath),
			dataType: "text",
			beforeSend: core.applyRoutingHeaders,
			cache: false,
			success: function(response) {
				deferred.resolve(response);
			},
			error: function() {
				deferred.reject();
			}
		});
		return deferred;
	};
	
	
	var retrieveFileMetadata = function(filePath) {
		var deferred = new $.Deferred();
		var ajaxPromise = null;

		if(core.collectiveRoutingRequired()) {
			
			// Get metadata from collective member
			encodedCollectiveServerId = core.encodeServerId(core.getCollectiveServerId());
			ajaxPromise = $.ajax({
				url: "/ibm/api/collective/v1/servers/" + encodedCollectiveServerId + "/fileservice/" + encodeURIComponent(filePath),
				dataType: "json",
				success: function(response) {
					if(response.resolvedPath != null && response.readOnly != null) {
						var metadata = {
								"fileName": response.resolvedPath,
								"readOnly": response.readOnly.toString()
						};
						deferred.resolve(metadata);
					} else {
						deferred.reject();
					}
				},
				error: function(jqXHR, textStatus) {
					deferred.reject(textStatus);
				}
			});
			
			
		} else {
			
			// Get metadata from local server
			ajaxPromise = $.ajax({
				url: "/IBMJMXConnectorREST/mbeans/WebSphere%3Afeature%3DrestConnector%2Ctype%3DFileService%2Cname%3DFileService/operations/getMetaData",
				type: "POST",
				contentType: "application/json",
				data: "{\"params\":[{\"value\":\"" + filePath.replace(/\\/g, "\\\\") + "\",\"type\": \"java.lang.String\"},{\"value\": \"r\",\"type\":\"java.lang.String\"}],\"signature\":[\"java.lang.String\",\"java.lang.String\"]}",
				success: function(response) {
					if(response.value != null) {
						deferred.resolve(response.value);
					} else {
						deferred.reject();
					}
				},
				error: function(jqXHR, textStatus) {
					deferred.reject(textStatus);
				}
			});
		}
		deferred.abort = function() {
			if(ajaxPromise != null) {
				ajaxPromise.abort();
			}
		};
		return deferred;
	};
	
	
	var isPathWritable = function(path) {
		if(serverWritePaths != null) {
			for(var i = 0; i < serverWritePaths.length; i ++) {
				if(path.indexOf(serverWritePaths[i]) == 0) {
					return true;
				}
			}
		}
		return false;
	};
	
	
	var applyVariablesToFilePath = function(filePath) {
		for(var i = 0; i < serverVariables.length; i++) {
			var variableResolvedPath = serverVariables[i].resolvedPath;
			if(variableResolvedPath != null) {
				if(filePath.indexOf(serverVariables[i].resolvedPath) == 0) {
					return "${" + serverVariables[i].name + "}" + filePath.substring(variableResolvedPath.length);
				}
			}
		}
		return filePath;
	};
	
	
	var retrieveServerVariableValues = function() {
		var deferred = new $.Deferred();
		var counter = 0;
		serverVariables.forEach(function(variable) {
			//TODO: trailing slash shouldn't be included, it's added here due to a bug in the collective metadata service
			retrieveFileMetadata("${" + variable.name + "}/").done(function(metadata) {
				if(metadata.fileName.charAt(metadata.fileName.length - 1) == "/") {
					metadata.fileName = metadata.fileName.substring(0, metadata.fileName.length -1);
				}
				variable.resolvedPath = metadata.fileName;
			}).fail(function() {
				variable.resolvedPath = "UNDEFINED";
			}).always(function() {
				counter++;
				if(counter == serverVariables.length) {
					serverVariables.sort(function(a, b) {
						return b.resolvedPath.length - a.resolvedPath.length;
					});
					deferred.resolve();
				}
			});
		});
		return deferred;
	};
	
	
	var retrieveServerWritePaths = function() {
		var deferred = new $.Deferred();
		
		var url = null;
		var collectiveRoutingRequired = core.collectiveRoutingRequired();
		if(collectiveRoutingRequired) {
			encodedCollectiveServerId = core.encodeServerId(core.getCollectiveServerId());
			url = "/ibm/api/collective/v1/servers/" + encodedCollectiveServerId + "/fileservice";
		} else {
			url = "/IBMJMXConnectorREST/mbeans/WebSphere%3Afeature%3DrestConnector%2Ctype%3DFileService%2Cname%3DFileService/attributes/WriteList";
		}
		
		$.ajax({
			url: url,
			dataType: "json",
			success: function(response) {
				if(collectiveRoutingRequired) {
					serverWritePaths = response.writeDirs;
				} else {
					serverWritePaths = response.value;
				}
				deferred.resolve();
			},
			error: function() {
				deferred.reject();
			}
		});
		
		return deferred;
	};
	
	
	var retrieveSchemaFile = function() {
		var deferred = new $.Deferred();
		
		var localStorageKey = "com.ibm.ws.sce.cached_schema_";
		
		if(core.collectiveRoutingRequired()) {
			var serverId = core.getCollectiveServerId();
			localStorageKey = localStorageKey + "remote_" + serverId.substring(0, serverId.lastIndexOf(","));
		} else {
			localStorageKey = localStorageKey + "local_" + window.location.hostname;
		}
		
		var schemaFromLocalStorage = localStorage.getItem(localStorageKey);		
		if(schemaFromLocalStorage != null) {
			deferred.resolve(schemaFromLocalStorage);
		} else {
			retrieveSchemaFileFromServer().done(function(schemaFromServer) {
				var retry;
				do {
					retry = false;
					try {
						localStorage.setItem(localStorageKey, schemaFromServer);
					} catch(error) {
						if(localStorage.length > 0) {
							localStorage.removeItem(localStorage.key(0));
							retry = true;
						}
					}
				} while (retry);
				deferred.resolve(schemaFromServer);
			}).fail(function() {
				deferred.reject();
			});
		}
		return deferred;
	};
	
	
	var retrieveSchemaFileFromServer = function() {
		var deferred = new $.Deferred();
		var locale = navigator.languages? navigator.languages[0] : (navigator.language || navigator.userLanguage);
		
		// Ensure locale format is the one accepted by the server runtime
		locale = locale.replace("-", "_");
		
		if(core.collectiveRoutingRequired()) {
			
			// Get metadata from collective member
			var serverId = core.getCollectiveServerId();
			var hostName = serverId.substring(0, serverId.indexOf(","));
			encodedCollectiveServerId = core.encodeServerId(serverId);

			// Retrieve install dir
			$.ajax({
				url: "/ibm/api/collective/v1/servers/" + encodedCollectiveServerId
			}).done(function(data) {
				var installDir = data.wlpInstallDir;

				// Retrieve schema from remote server
				$.ajax({
					url: "/ibm/api/collective/v1/runtimes/" + hostName + "," + encodeURIComponent(installDir) + "/schema?locale=" + locale + "&compactOutput=true&forceGeneration=true",
					dataType: "text"
				}).done(function(schema) {
					deferred.resolve(schema);
				}).fail(function() {
					deferred.reject();
				});
			}).fail(function() {
				deferred.reject();
			});			
			
		} else {
			
			// Retrieve schema from local server
			$.ajax({
				url: "/IBMJMXConnectorREST/mbeans/WebSphere%3Aname%3Dcom.ibm.ws.config.serverSchemaGenerator/operations/generateInstallSchema",
				type: "POST",
				contentType: "application/json",
				data: "{\"params\":[{\"value\": null, \"type\": \"java.lang.String\" },{\"value\": null, \"type\": \"java.lang.String\" },{\"value\": null, \"type\": \"java.lang.String\" },{\"value\": \"" + locale + "\", \"type\": \"java.lang.String\" }, {\"value\": \"true\", \"type\": \"java.lang.Boolean\"}], \"signature\": [ \"java.lang.String\", \"java.lang.String\", \"java.lang.String\", \"java.lang.String\", \"boolean\" ]}",
				success: function(responseData) {
					var filePath = responseData.value.keyFilePath;
					$.ajax({url: "/IBMJMXConnectorREST/file/" + encodeURIComponent(filePath),
						dataType: "text",
						cache: false,
						success: function(responseData) {
							// Delete temporary file
							$.ajax({
								url: "/IBMJMXConnectorREST/file/" + encodeURIComponent(filePath),
								type: "DELETE",
							});
							deferred.resolve(responseData);
						},
						error: function() {
							deferred.reject();
						}
					});
				}, error: function(data) {
					deferred.reject();
				}
			});
		}

		return deferred;
	};
	
	var isServerReadOnly = function() {
		return serverWritePaths == null || serverWritePaths.length == 0;
	};
	
	
	var getLastSlashIndex = function(value) {
		var index = value.lastIndexOf("/");
		if(index == -1) {
			index = value.lastIndexOf("\\");
		}
		return index;
	};
	
	
	getPathFromFilePath = function(filePath) {
		return filePath.substring(0, getLastSlashIndex(filePath));
	};
	
	
	var getFileFromFilePath = function(filePath) {
		return filePath.substring(getLastSlashIndex(filePath) + 1);
	};

	
	return {
		retrieveFile: retrieveFile,
		retrieveFileContent: retrieveFileContent,
		retrieveServerVariableValues: retrieveServerVariableValues,
		retrieveServerWritePaths: retrieveServerWritePaths,
		retrieveSchemaFile: retrieveSchemaFile,
		resolveIncludeLocations: resolveIncludeLocations,
		resolveIncludeLocation: resolveIncludeLocation,
		applyVariablesToFilePath: applyVariablesToFilePath,
		isServerReadOnly: isServerReadOnly,
		getPathFromFilePath: getPathFromFilePath,
		getFileFromFilePath: getFileFromFilePath,
		isPathWritable: isPathWritable
	};

})();
